// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnlibs.hpp"
#include "display/Transform.hpp"

namespace Dawn {
  class SceneItem;
  class Scene;
  class DawnGame;

  class SceneItemComponent {
    public:
      SceneItem &item;
      Transform &transform;

      /**
       * Constructs a new SceneItemComponent. Components are attached to
       * SceneItems and will be individual responsibility components, and must
       * communicate to other items/components using methods and events.
       * 
       * @param item Scene Item thsi component belongs to.
       */
      SceneItemComponent(SceneItem &item);
      
      /**
       * Requested on the first frame that the parent scene item has become
       * active.
       */
      virtual void start() = 0;

      /**
       * Shorthand to return the scene that this component's item belongs to.
       * @return The current scene.
       */
      Scene & getScene();

      /**
       * Shorthand to return the game that this scene belongs to.
       * @return The current game.
       */
      DawnGame & getGame();

      /**
       * Cleanup the SceneItemComponent.
       */
      virtual ~SceneItemComponent();
  };
}